import java.awt.Point;

// ******************************************************************************
//			Block
// ******************************************************************************

// Questa classe implementa la gestione dei PEZZI.	
// Fornisce le primitive :	
//  1 :  Assegnazione di un pezzo a un giocatore.
//  2 :  Verifica che un giocatore possiede un pezzo.
//  3 :  Sottrazione di un pezzo a un giocatore.

// Nel server i pezzi vengono distribuiti fra i giocatori "veri".
// Nei cliente i pezzi vengono distribuiti fra il giocatore assegnato al client e
// un giocatore fittizzio.
// Il giocatore fittizzio rappresenta l'insieme degli avversati
// Al giocatore fittizzio vengono assegnati tutti i pezzi posseduti dagli avversari

// Il termne PEZZO viene utilizzato con l'usuale significato.
// Un pezzo e' formato da due FIGURE.

// Un pezzo viene indivato con una variabile di tipo Point.
//   Point.x indica la prima figura	[ 0..numblock-1 ]
//   Point.y indica la seconda figura	[ 0..point.x ]
//   Se y>x le due coordinate vengono automaticamente scambiate



public class Block {

  static public final int numblock = 7;	

	// Indica il numero di possibili figure ( 0..numblock-1 )

  private int numplayer;	
	
	// Indica il numero di giocatori ( 1..numplayer )  

  private int data[][];	

	// Indica il giocatore a cui e' assegnato il pezzo (i,j)
	// Assume :
	//      -1 		pezzo posizionato sul tavolo
	//       0		pezzo appartenente a giocatore fittizzio
	//   1..numplayer	pezzo appartenete a un giocatore   


  public Block() {

	// Costruttore :
	// Il numero di giocatori deve essere specificato all'inizio della
	// partita, il costruttore gli assegna -1 per indicare  deve essere 
	// ancoa definito.  

    numplayer=-1;	
    data = new int[numblock][numblock];
  }


  public void setNumPlayer( int numplayer ) {

	// Imposta, se non e' gia stato definito, il numero di giocatori 

    if ( this.numplayer == -1 ) this.numplayer = numplayer;
  }


  public int getNumPlayer() {

	// Restituisce il numero di giocatori

   return numplayer;
  }


  private Point MaxMin( Point point ) {

	// Se point.y>point.x le due coordinate vengono scambiate
     
    Point pt;   
    if ( point.y<=point.x ) pt=new Point(point.x,point.y);
    else                    pt=new Point(point.y,point.x);
    return pt;
  }


  public int putBlock( Point block , int player ) {

	// Assegna un pezzo ad un giocatore.
	
	// Restituisce :
	//   0 : Nessun Errore
	//  -1 : Pezzo non esistente
	//  -2 : Pezzo gia' assegnato

	// Attiva la procedura di scambio delle coordinate
    Point pt=MaxMin( block );

	// Verifica correttezza Pezzo
    if (( pt.x<0 ) || ( pt.y<0 ) || ( pt.x>=numblock )) return -1;

	// Verifica correttezza numero giocatore.
	// E' un errore che si puo' verificare solo nel caso che il codice
	// sia scritto non correttamente, il programma viene interrotto.
    if (( player>numplayer ) || ( player<0 )) {
      System.out.println("Block.putBlock : Giocatore ("+player+"#"+numplayer+") non valido");
      System.exit(1);
    }
	// Verifica che il pezzo sia gia' assegnato
    if ( data[pt.x][pt.y] != 0 ) return -2;

    data[pt.x][pt.y]=player;
    return 0;		// Nessun errore
  }


  public int getBlock( Point block , int player ) {

	// Verifica se un giocatore possiede un pezzo

	// Restituisce :
	//   0 : Il pezzo appartiene al giocatore
	//  -1 : Pezzo non esistente
	//  -2 : Il pezzo non appartiene al giocatore

    Point pt=MaxMin( block );

    if (( pt.x<0 ) || ( pt.y<0 ) || ( pt.x>=numblock )) return -1;
    if (( player>numplayer ) || ( player<0 )) {
      System.out.println("Block.getBlock : Giocatore ("+player+"#"+numplayer+") non valido");
      System.exit(1);
    }
    if ( data[pt.x][pt.y] != player ) return -2;
    return 0;
  }


  protected int delBlock( Point block , int player ) {

	// Restituisce un codice di errore :
	//
	//   0 : Nessun Errore
	//  -1 : Pezzo non esistente
	//  -2 : Pezzo non appartenente al giocatore

    Point pt=MaxMin( block );

    if (( pt.x<0 ) || ( pt.y<0 ) || ( pt.x>=numblock )) return -1;
    if (( player>numplayer ) || ( player<0 )) {
      System.out.println("Block.delBlock : Giocatore ("+player+"#"+numplayer+") non valido");
      System.exit(1);
    }      
    if ( data[pt.x][pt.y] != player ) return -2;
    data[pt.x][pt.y]=0;
    return 0;
  }   

}  // class
