import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import MoveState;
import cnvIOMap;
import cnvIOBlock;
import ClientGame;

// ***********************************************************************************
//			ClientFrame
// ***********************************************************************************

// Gestisce la parte grafica del Client, 
// Fornisce primitive per : 
//   1  Visualizzare i pezzi posseduti dal giocatore
//   2  Visualizzare i pezzi gia' posizionati nel tavolo
//   3  Visualizzare i nomi di tutti i giocatori 
//   4  Visualizzare Quote e le comunicazioni con il Server
//   5  Visualizzare eventuali errori
//   6  Permettere al giocatore di richiedere le operazioni di Move, Quote , Exit


class ClientFrame implements ActionListener {


  private Frame mainFrame;
  private Frame	outputFrame;

	// Finestra Principate e finestra per Output Quote e 
	// comunicazioni con il Server

  private ClientGame game;

	// Puntatore alla classe ClientGame, operazioni richieste dal giocatore 
	// vengono convertite in chiamate alla classe ClientGame

  private MoveState   movestate;
  private cnvIOMap    cnvmap;
  private cnvIOBlock  cnvblock;

	// Gestiscono il tavolo dei pezzi posizionati e il tavolo dei pezzi
	// posseduti dal giocatore
    
  private Button 	bMove;		// Pulsanti Move, Quote, Exit
  private Button	bQuote;
  private Button	bExit;
  final String 		sStart = "Start";
  final String 		sMove  = "Move";

  Label label[];			// Label per la visualizzazione dei nomi dei giocatori

  private TextArea  textErr;		//  Aree di Testo per Output Errori e Quote
  private TextArea  textQuote;

  private Frame 	QuoteFrame;	// Frame per input Quote
  private Button 	bQuoteSend;
  private Button 	bQuoteAbort;
  private TextField	tQuote;

  private Frame		ErrFrame;	// Frame per Output messaggi importanti
  private Button	bErrOk;
  private Label		lErr;

  private Image Img[];			// Immagini dei Pezzi

  private Panel pnl;


  public ClientFrame( Image Img[], ClientGame game) {

	// Costruttore, definisce l'ambiente grafico nella sola parte di output
	// dei nomi dei giocatori, ovvero nella fase fra l'aver dato il proprio nome e
	// l'inizio della partita

    this.game = game;
    this.Img = Img;

    mainFrame = new Frame("Main Windows");
    mainFrame.setSize(680,340);

    mainFrame.setLayout( new GridLayout(1,2,10,10));	 // O|O

    pnl = new Panel();
    pnl.setLayout( new GridLayout(2,1,10,10));		// O/O
    mainFrame.add(pnl);

    Panel pnl1 = new Panel();
    pnl1.setLayout( new GridLayout(7,1,2,2));		// O/O/O/O/O/O
    pnl.add(pnl1);

	// Inizializza le Label dedicate all'output dei nomi dei giocatori

    Font font= new Font("MY",Font.PLAIN,14);
    label = new Label[6];
    label[0]=new Label("Player :" );
    label[0].setFont( font );
    pnl1.add( label[0] );    

    for ( int i = 1 ; i<6 ; i++ ) {
      label[i] = new Label();    
      label[i].setFont( font );
      pnl1.add( label[i] );    
    }  

    Panel pnl2 = new Panel();
    pnl2.setLayout( new GridLayout(1,3,10,10));		// O|O|O
    pnl1.add(pnl2);

    bMove  = new Button(sStart);
    bQuote = new Button("Quote");
    bExit  = new Button("Exit");
    pnl2.add( bMove );
    pnl2.add( bQuote );
    pnl2.add( bExit );
    bMove.addActionListener(this);
    bQuote.addActionListener(this);
    bExit.addActionListener(this);

    mainFrame.setVisible( true );

	//  Crea frame di Output Errori e Quote

    outputFrame = new Frame("Text Windows"); 
    outputFrame.setSize(680,250);
    outputFrame.setLayout( new GridLayout(0,2,0,0));
    textErr = new TextArea("Questa e' la finestra per l'Output degli Errori \n");
    outputFrame.add( textErr);     
    textQuote = new TextArea("Questa e' la finestra per l'Output dei Quote \n");
    outputFrame.add( textQuote);     
    outputFrame.setVisible( true );

	// Crea frame di Input Quote

    QuoteFrame  = new Frame("Quote Windows"); 
    QuoteFrame.setSize(420,140);
    QuoteFrame.setResizable( false );
    QuoteFrame.setLayout( new FlowLayout() );
    QuoteFrame.add( new Label("    Enter your Quote    "));
    tQuote = new TextField("",45);
    QuoteFrame.add( tQuote ); 
    bQuoteSend  = new Button("Send");
    bQuoteAbort = new Button("Abort");
    QuoteFrame.add( bQuoteSend );
    QuoteFrame.add( bQuoteAbort );
    bQuoteSend.addActionListener(this);
    bQuoteAbort.addActionListener(this);
    QuoteFrame.setVisible( false );

	// Crea frame per output messaggi importanti

    ErrFrame = new Frame("Error Windows");
    ErrFrame.setSize(420,140);
    ErrFrame.setResizable( false );
    ErrFrame.setLayout( new GridLayout(2,0,0,0) ); 
    font= new Font("MY",Font.PLAIN,14);

    Panel pnl3 = new Panel();   
    pnl3.setLayout( new FlowLayout() );
    ErrFrame.add(pnl3);
    lErr = new Label();
    lErr.setFont( font );
    pnl3.add(lErr);

    Panel pnl4 = new Panel();   
    pnl4.setLayout( new FlowLayout() );
    ErrFrame.add(pnl4);
    bErrOk  = new Button("  Ok  ");
    bErrOk.setFont( font );
    pnl4.add( bErrOk );
    bErrOk.addActionListener(this);
    ErrFrame.setVisible( false );
  }


  public void createTable() {

	// Questo metodo viene chiamato dalla classe ClientGame quando arriva
	// il comando START "inizio partita" dal Server.
	// Aggiunge all'ambiente grafico i tavoli per la visualizzazione dei
	// pezzi posseduti dal giocatore e quelli posizionati

    bMove.setLabel(sMove);
    bMove.setEnabled( false );

    movestate = new MoveState(this, game.getPlayer());
    cnvmap    = new cnvIOMap(game, movestate, Img);
    cnvblock  = new cnvIOBlock(game, movestate, Img);

    mainFrame.add(cnvmap);  
    pnl.add(cnvblock);    

    mainFrame.show();
  } 


  public void paint() {

	// Aggiorna l'ambiente grafico

    cnvmap.repaint();
    cnvblock.repaint();
  }


  public void setEnabledMove( boolean flag ) {

	// Abilita e disabilita il pulsante Move

    bMove.setEnabled( flag );
  } 


  public void setEnabledQuote( boolean flag ) {

	// Abilita e disabilita il pulsante Quote

    bQuote.setEnabled( flag );
  } 


  public void appErr( String str ) {

	// Aggiunge la stringa 'str' alla finestra di testo 'textErr'

    textErr.append( str );
  }


  public void appQuote( String str ) {

	// Aggiunge la stringa 'str' alla finestra di testo 'textQuote'

    textQuote.append( str );
  }


  public void setLabel( int numLabel , String str ) {

	// Definisce la Label 'numlabel', viene utilizzata per l'output dei nomi 
	// dei giocatori

    label[numLabel].setText( str );
  }


  public void cleanLabel( int numLabel ) {

	// Cancella la Label 'numLabel', e sposta verso l'alto di una posizione
	// le label sottostanti, viene utilizzato quando un giocatore si ritira 
	// prima dell'inizio della partita

    int i;
    for ( i=numLabel ; ( i<5 ) && (!label[i+1].getText().equals(""))  ; i++ )
      label[i].setText(" -"+i+"- "+label[i+1].getText().substring(4));      

    label[i].setText(""); 
  }


  public void outErr( String str ) {

	// Apre la finestra 'ErrFrame' e visualizza il messaggio 'str'

    lErr.setText(str);
    ErrFrame.setVisible(true);
  }


  public void actionPerformed(ActionEvent e ) {

	// Gestisce le operazioni richieste dal giocatore, ovvero Move, Quote, Exit

    String cmd = e.getActionCommand();

	// Pulsante START

    if ( cmd.equals(sStart) ) 
      game.writeSTART();

	// Pulsante MOVE

    else if ( cmd.equals(sMove) ) {
      if ( movestate.block != null )  
        game.writeMOVE(	movestate.block.x, movestate.block.y, 
			movestate.point.x, movestate.point.y, movestate.mode);
      else bMove.setEnabled( false );
    }
	// Pulsante Quote

    else if ( cmd.equals(bQuote.getLabel()) ) QuoteFrame.setVisible( true ); 

    else if ( cmd.equals(bQuoteSend.getLabel()) ) {
      QuoteFrame.setVisible( false ); 
      game.writeQUOTE( tQuote.getText());
    }
    
    else if ( cmd.equals(bQuoteAbort.getLabel()) )  QuoteFrame.setVisible( false );                   

	// Pulsante EXIT

    else if ( cmd.equals(bExit.getLabel()) ) {
      game.doDEAD(null);
      mainFrame.dispose();
      outputFrame.dispose(); 
      QuoteFrame.dispose();
      ErrFrame.dispose();          
    }
	// Pulsante ErrFrame.Ok

    else if ( cmd.equals(bErrOk.getLabel()) ) ErrFrame.setVisible(false); 
  }
      
}  // class