import java.awt.Point;
import java.io.IOException;
import java.net.*;
import java.io.*;
import Parser;
import ClientGame;

// ********************************************************************************
//				ClientThread
// ********************************************************************************

// Implementa la gestione delle comunicazini fra Client e Server
// Fornisce un Thread il cui codice e' definito nel metodo run() che attende 
// ed esegue i comandi provenienti dal Server.
// Fornisce primitive per inviare comandi al Server.
// I metodi non sono definiti mutuamente esclusivi ( synchronized ) in quanto
// ogni comando viene convertito in una chiamata alla classe ClientGame il cui
// acesso e' mutuamente esclusivo.


class ClientThread implements Runnable {

  private  ClientGame game;

	// Puntatore alla classe ClientGame, ogni comando viene convertito
	// in una chiamata alla classe ClientGame.

  private Parser parser;

	// Interprete dei comandi dal Server

  private Socket socket;
  private BufferedReader is;
  private PrintWriter    os;

	// Socket e Stream di input/output


  public ClientThread ( Socket socket , ClientGame game ) throws IOException {

	// Costruttore
	//   socket	: Socket con il Server
	//   game	: Puntatore alla classe ClientGame

    this.game   = game;		// Copia delle variabili locali
    this.socket = socket;

	// Crea stream di input ed output

    is = new BufferedReader( 
             new InputStreamReader (
                 new BufferedInputStream (   
                     socket.getInputStream() ) ) );

    os = new PrintWriter( socket.getOutputStream());

	// Definisce e crea interprete di comandi dal Server

    int numComm = 10;  		// Numero di comandi
    String dataComm[];
    dataComm = new String[numComm];

	// Definizione Comandi
 
    dataComm[0] = new String("HELO");
    dataComm[1] = new String("NAME Integer,String");
    dataComm[2] = new String("CLEAN Integer");
    dataComm[3] = new String("PLAY Integer,Integer");
    dataComm[4] = new String("BLOCK Integer,Integer");
    dataComm[5] = new String("START");
    dataComm[6] = new String("MOVE Integer,Integer,Integer,Integer,Integer,Integer");
    dataComm[7] = new String("GO"); 
    dataComm[8] = new String("QUOTE Integer,String");
    dataComm[9] = new String("END");

    parser = new Parser( dataComm , numComm );
  }


  public void run() {

	// Il metodo run() contiene il codice eseguito dal Thread dedicato 
	// alle comunicazioni in ingresso dal Server.
	// Attende un comando dal Server, lo analizza, e se corretto esegue 
	// la corretta chiamata alla Classe ClientGame.
	// Il thread viene terminato quando il socket viene chiuso o dal server o
	// da una chiamata della classe ClientGame. 

    String str;			// Comando letto dal Socket

    do {

      try {

        str = is.readLine();	// Legge un comando dal Socket

      } catch (IOException e) { 

		// In caso di IOException la partita viene terminata
        game.doDEAD( e );
	return;
      }   
      if ( str == null ) {
		// Questa condizione viene tornata quando in Socket e'
		// stato chiuso correttamante dal Server, la partita viene
		// terminata
	game.doDEAD( new IOException("Socket closed to Server"));
	return;
      }
  
      switch ( parser.setCommand(str) ) {	// Analizza ed esegue il comando

        case 0 :	// HELO 
          game.doHELO ();
          break;

	case 1 :	// NAME
	  game.doNAME ( parser.getInteger(0).intValue(), parser.getString(1) );
	  break;

	case 2 :	// CLEAN
	  game.doCLEAN( parser.getInteger(0).intValue()  );
	  break;

	case 3 :	// PLAY
	  game.doPLAY ( parser.getInteger(0).intValue(), parser.getInteger(1).intValue() ); 
	  break;

	case 4 :	// BLOCK
          game.doBLOCK ( parser.getInteger(0).intValue(), parser.getInteger(1).intValue() ); 
          break;

        case 5 :	// START
          game.doSTART ();
          break;

        case 6 : 	// MOVE
          game.doMOVE ( parser.getInteger(0).intValue(), parser.getInteger(1).intValue(), 
			parser.getInteger(2).intValue(), parser.getInteger(3).intValue(), 
			parser.getInteger(4).intValue(), parser.getInteger(5).intValue() );
          break;

	case 7 :	// GO
	  game.doGO();
	  break;

        case 8 :	// Nuovo QUOTE
          game.doQUOTE ( parser.getInteger(0).intValue(), parser.getString(1) );
          break;

	case 9 :	// Comando END
	  game.doEND();
	  break;


	default :
	  game.doERROR("Parser Error "+parser.getCommand()+" : "+parser.toString() ); 
      } 
    	      
    } while ( 1==1 ); 

  }


// I metodi write* vengono chiamati dalla classe ClientGame per inviare messaggi
// al Server,  e' definito un metodo per ogni comando valido che puo' essere 
// inviato.
// In caso di errore di scrittura sul Socket viene generato un IOException che
// deve essere intercettato nella classe ClientGame.

  public void writeSTART () throws IOException {

  	// Invia al Server il Comando START
  
    String command = "START";
    os.println( command );

    if ( os.checkError() ) 
      throw new IOException("ClientThread.writeSTART");
  }


  public void writeQUOTE ( String quota ) throws IOException {

  	// Invia al Server il comando QUOTE 
  
    String command = "QUOTE " + quota;
    os.println( command );

    if ( os.checkError() ) 
      throw new IOException("ClientThread.writeQUOTE");
  }


  public void writeMOVE ( int pz1,  int pz2, 
			  int pnt1, int pnt2, int rot ) throws IOException {

  	// Invia al Server il comando MOVE 
  
    String command = "MOVE "+pz1+","+pz2+","+pnt1+","+pnt2+","+rot;
    os.println( command );

    if ( os.checkError() ) throw new 
      IOException("ClientThread.writeMOVE");
  }


  public void writeNAME ( String name ) throws IOException {

  	// Invia al Server il comando NAME
  
    String command = "NAME " + name;
    os.println( command );

    if ( os.checkError() ) 
      throw new IOException("ClientThread.writeNAME");
  }


  public void disableThread() {

	// Chiude gli stream di I/O ed il Socket.

    try {
      os.close();
      is.close();
      socket.close();    
    } catch (IOException e) {}  
  }


} // class
