import java.awt.Point;
import Block;

// ****************************************************************************
//			Map
// ****************************************************************************

// Implementa la gestione del tavolo su cui si posizionano i PEZZI.
// Fornisce le primitive per :

//  1 : Aggiornare e verificare lo stato della partita
//  2 : Posizionare un pezzo nel tavolo
//  3 : Conoscere la figura posizionata in una data posizione del tavolo.


public class Map extends Block {

  private int state;

	// Indica lo stato della partita
        //   0 : La partita deve ancora incominciare 
	//   1 : Partita iniziata deve errere effettuata la prima mossa
	//   2 : Partita Iniziata prima mossa effettuta
 
  private int data[][];

	// Indica quale figura si trova nella posizione (i,j) del tavolo
	//
	//  -1 			posizione vuota
	// 0..block.numblock-1	indica la figura
	//
	// 0 <= i <= size.x-1
	// 0 <= j <= size.y-1

  public Point size;

	// Dimensione del tavolo

  private Point pose1,pose2;

	// Indicano la posizione dei due estremi nel tavolo

  private int fige1,fige2; 

	// Indicano il tipo di figura che si trovano nei due estremi

  static public final int nummode=4;
  static public final int modetype[][]={ { -1 , 0 } , { 0 , 1 } , { 1 , 0 }  , { 0 , -1 } };

	// Indica il modo con cui possono essere posizionati i pezzi 
	// Seguono un ordinamento orario :
	//   Alto - Destra  - Basso - Sinistra

  
  public Map( Point size ) {

	// Costruttore

    super();		// Chiama il costruttore della classe Block
    state = 0;		// Imposta lo stato a 0 "La partita deve ancora iniziare"

	// Imposta ogni posizione del tavolo a -1 "Posizione vuota"
 
   this.size = new Point(size.x,size.y);
    data = new int[size.x][size.y]; 
    for ( int i=0 ; i < size.x ; i++ )
      for ( int j=0 ; j < size.y ; j++ )
        data[i][j]=-1;

    pose1 = new Point(0,0);
    pose2 = new Point(0,0);
    fige1 = 0;
    fige2 = 0;
  }


  public void startPlay() {

	// Se la partita deve ancora incominciare imposta lo stato a
	// 1 "Partita iniziata deve errere effettuata la prima mossa"

    if ( state == 0 ) state = 1;  
  }


  public int getState() {

	// Restituisce lo stato della partita

    return state;
  }


  public int getExtreme( int extreme ) {

	// Restituisce la figura nell'estremo "extreme"

    if ( extreme == 1 )	return fige1;
    else 		return fige2;
  }


  public int check( int player, Point block, Point point, int mode ) {

	// Verifica se il giocatore "player" puo' muovere il pezzo
	// "block" nella posizione "point" con rotazione "mode"
	//
	// Restituisce :
	//
	//  12 : NoE NoPM ( Estremo 2 Figura Y )	
	//  11 : NoE NoPM ( Estremo 1 Figura Y )
	//   2 : NoE NoPM ( Estremo 2 Figura X )
	//   1 : NoE NoPM ( Estremo 1 Figura X )
	//   0 : NoE Prima Mossa 
	//  -1 : Pezzo non esistente
	//  -2 : Il pezzo non appartiene al giocatore
	//
	//  -4 : "point" non adiacente ad un estremo 
	//  -5 : La figura nell'estremo non coincide con il pezzo
	//  -6 : La pezzo si sovrappone ad altri pezzi
	//  -7 : Il pezzo esce dalla mappa
        //  -8 : La partita deve ancora incominciare        

    int pose = 0;
    int fige = 0;
    int err = 0;

    mode = mode % nummode;	// mode viene considerato in modulo

	// Verifica se la partita deve ancora iniziare ( Errore -8 )

    if ( state == 0 ) return -8;

	// Verifica se il pezzo esiste e se appartiene al giocatore ( Errore -1,-2 )

    err = getBlock( block , player );
    if ( err!=0 ) return err;

    try {

	// Verifica che il pezzo non si sovrapponga con gli altri pezzi ( Errore -6 ) 

      if ( data[point.x][point.y]!=-1 ) return -6;
      if ( data[point.x+modetype[mode][0]][point.y+modetype[mode][1]]!=-1 ) return -6;

    } catch ( IndexOutOfBoundsException e ) {

	// Verifica che la figura sia posizionata all'interno della ( mappa -7 )

      return -7;
    }   

	// Se e' la prima mossa non servono altri controlli

    if ( state == 1 ) return 0;

	// Se non e' la prima mossa

	// Verifica che 'point' sia adiacente ad un estremo ( Errore -4 )
	// cercando una ugualianza fra la posizione dell'estremo e la posizione 
	// dei vicini di "point"

    err=-4;
    for ( int i =0 ; ( i<nummode ) && ( err==-4 ) ; i++ ) {
      if ( pose1.equals(new Point( point.x+modetype[i][0], point.y+modetype[i][1] ))) {
        err = 0 ;
        pose = 1;
        fige = fige1;
      } 
      if ( pose2.equals(new Point( point.x+modetype[i][0], point.y+modetype[i][1] ))) {
        err = 0 ;
        pose = 2;
        fige = fige2;
      } 
    }
    if (err == -4) return err;

	// 'fige' contiene il tipo di figura nell'estremo indicato
	// 'pose' indica l'estremo (  1 o 2 ) a cui il pezzo e' adiacente

	// Verifica che la figura nell'estremo coincida con il pezzo ( Errore -5 )
	// Se corretta aggiorna pose

    if ( fige == block.y ) pose=pose+10;
    else if ( fige!=block.x) return -5;

    return pose;
  }


  public int putMap( int player, Point block, Point point, int mode ) {

	// Il giocatore "player" posizione il pezzo
	// "block" nella posizione "point" con rotazione "mode"
	//
	// Restituisce :
	//
	//   0 : Nessun Errore
	// < 0 : Come "check" 

    int fige,pose;

    mode = mode % nummode;	// mode viene considerato in modulo
    pose = check(player ,block , point , mode ); 

    if ( pose<0 ) return pose;
                
	// Altrimenti Parametri corretti......Posiziona il pezzo

    if ( state == 1 ) {

	// Prima mossa
	// Posiziona il pezzo e' imposta 'fige'  'pose' 

      pose1.move(point.x,point.y);
      pose2.move(point.x + modetype[mode][0], point.y + modetype[mode][1] );
      fige1 = block.x;
      fige2 = block.y;

      data[pose1.x][pose1.y] = fige1;
      data[pose2.x][pose2.y] = fige2;

	// Imposta lo stato a 2 ( Partita Iniziata prima mossa effettuta )

      state = 2;

    } else {

	// Altrimenti:
	//	(pose mod 10) indica l'estremo ( 1 o 2 ) adiacente al pezzo
	//	(pose div 10) indica quale delle due figure del pezzo
	//		      corrisponde a quella contenuta nell estremo.

	// Posizione la prima meta' del pezzo

      if ( pose>10) {	// Figura estremo corrisponde con figura y
	data[point.x][point.y] = block.y;	
	fige = block.x;
      } else {		// Figura estremo corrisponde con figura x
	data[point.x][point.y] = block.x;
	fige= block.y;
      }

	// 'fige' contiene la nuova figura indicata dall'estremo
	// Posiziona l aseconda meta' del pezzo

      data[point.x+modetype[mode][0]][point.y+modetype[mode][1]]=fige;

	// Aggiorna 'pose' , 'fige'

      if ( pose % 10 == 1 ) {
        pose1.move(point.x+modetype[mode][0],point.y+modetype[mode][1]);
        fige1 = fige;
      } else {
        pose2.move(point.x+modetype[mode][0],point.y+modetype[mode][1]);
        fige2 = fige;
      }
    }  
	// Toglie il pezzo mosso al giocatore

    if ( delBlock( block , player ) != 0 )
	// Si puo' verificare solo nel caso che il codice sia scritto
	// non correttamente, il programma viene interrotto.
      throw new Error("Map.putMap : Errore Ingestibile");

    return 0;
  }


  public int getMap( Point point ) { 

	// Restituisce :
	//
	// >=0 : Figura presente nel tavolo alla posizione 'point' 
	//  -1 : Posizione vuota

    return data[point.x][point.y];
  }


  public String errorString( int e ) {

	// Restituisce una stringa che descrive l'errore specificato

    if ( e >= 0 ) return "Nessun Errore";
    switch ( e ) {
    case -1 :	return "Pezzo non esistente";
    case -2 :	return "Pezzo non appartenente al giocatore";
    case -4 :	return "Posizione Pezzo non adiacente ad un estremo"; 
    case -5 :	return "Le figure dell'estremo e del Pezzo non coincidono";
    case -6 : 	return "Il Pezzo si sovrappone ad altri pezzi";
    case -7 : 	return "Posizione Pezzo esterno alla mappa";
    case -8 :	return "La partita deve ancora incominciare";
    default :	return "Errore: "+e+" non definito";
    }
  }
    

}  // class


   

