import java.net.*;
import java.io.*;
import ServerGame;
import java.util.Date;
import java.text.SimpleDateFormat;
import ServerLog;
import ServerCounter;
import java.lang.reflect.Array;


// ***************************************************************************
//			Server
// ***************************************************************************

// questa classe contiene il main ed esegue lo spawning dei vari giochi e
// gestisce le connessioni al socket.


class Server {

	// un po' di valori di default per il parsing...
  private static final int minPort	= 4444;
  private static final int defPort	= 4444;
  private static final int defLevel	= 0;
  private static final int MAXGames	= 1024;
  private static final int minGames	= 16;
  private static final String defLogFile = new String("/var/log/domino.log");



	// esegue il parsing delle opzioni in linea di comando, e inoltre
	// crea il ServerSocket e gestisce le connessioni, passandole al gioco
	// attivo in quel momento

  public static void main ( String args[] ) {

    int	level	= defLevel;
    int port	= defPort;
    int max	= minGames;
    String file = null;


	// eseguo il parsing delle opzioni
    for ( int i = 0 ; i< Array.getLength(args) ;  ) {
  
      if (args[i].compareTo("-l") == 0) {
        if ( (i+1) == Array.getLength(args) ) {
          System.out.println("Wrong options value, Server -h for help");
          System.exit(1);
        }
        
        try {
          level = Integer.valueOf(args[i+1]).intValue(); }
        catch (NumberFormatException e) {
          System.out.println("Argument not a number, Server -h for help");
          System.exit(1);
        }

        if ((level < 0) || (level > 10)) {
          System.out.println("Log level out of range, Server -h for help");
          System.exit(1);
        }
        i = i+2;
      } else
          if ( args[i].compareTo("-p") == 0 ) {
            if ( (i+1) == Array.getLength(args) ) {
              System.out.println("Wrong options value, Server -h for help");
              System.exit(1);
            }
            
            try {
              port = Integer.valueOf(args[i+1]).intValue(); }
            catch (NumberFormatException e) {
              System.out.println("Argument not a number, Server -h for help");
              System.exit(1);
            }
        
            if (port < minPort ) {
              System.out.println("TCP port out of range, Server -h for help");
              System.exit(1);
            }
            i = i+2;
          } else 
              if ( args[i].compareTo("-g") == 0 ) {
                if ( (i+1) == Array.getLength(args) ) {
                  System.out.println("Wrong options value, Server -h for help");
                  System.exit(1);
                }

                try {
                  max = Integer.valueOf(args[i+1]).intValue(); }
                catch (NumberFormatException e) {
                  System.out.println("Argument not a number, Server -h for help");
                  System.exit(1);
                }

                if ((max < minGames) || (max > MAXGames)) {
                  System.out.println("Number of max allowed games out of range, Server -h for help");
                  System.exit(1);
                }
                i = i+2;
              } else
                  if ( args[i].compareTo("-h") == 0 ) {
                    printHelp();
                    System.exit(0);
                  } else {
                      printHelp();
                      System.exit(1);
                  }
    }

	// definisco il livello di logging
    ServerLog.setLevel(level);
    	// definisco il numero massimo di giochi contemporanei
    ServerCounter.setMaxGames(max);

	// la partita 0  creata di default
    int		 numGame 	= 1;
    ServerSocket serverSocket 	= null; 
    Socket	 socket 	= null;

	// Crea un ServerSocket
    ServerLog.doLog(0,"started");

    try {
      serverSocket = new ServerSocket(port);
    } catch (IOException e) {
      ServerLog.doLog(0,"ServerSocket failed: " + e);   
      System.exit(1);                 
    }
    ServerLog.doLog(0,"ServerSocket created on port " + port); 


	// Crea una Partita
    ServerGame	 game = new ServerGame(0);
    ServerCounter.addGame();


	// cicla e accetta connessioni. Tenta di passarle poi al gioco di
	// cui ha correntemente il puntatore; se fallisce ne crea un altro.
    int	isOk;

    do {
 
      isOk = 0;

	// Si mette in attesa di una connessione
      try {
        socket = serverSocket.accept(); 
      } catch (IOException e) {
        ServerLog.doLog(2,numGame,0,"accept failed: " + e);
        isOk = -1;
      }	
	// Se la connessione e' avvenuta in modo corretto
	// viene passato il Socket al Gioco
      if ( isOk==0 ) 
      		// se il socket non viene accettato devo creare un nuovo gioco
      	if ( game.acceptSocket( socket ) != 0 ) {
      	  if ( ServerCounter.addGame() == 0 ) {
	    game = new ServerGame( numGame );
	    numGame = (numGame+1)%(MAXGames*16);
	    if ( game.acceptSocket( socket ) != 0 ) {
	      ServerLog.doLog(0,"Server.java: code error, please debug");   
	      System.exit(1);                 
	    }
		// se non posso creare giochi, chiudo il socket senza
		// tanti complimenti (la porta in faccia... ;)))
	  } else {
	      ServerLog.doLog(3,"Server.java: client refused, games full");
	      try {
	        socket.close();
	      } catch (IOException e) {}
	  }
	    
	}
    } while ( 1==1 );

  }  // main


	// metodino che stampa un help sommario

  private static void printHelp () {
  
    System.out.println("Domino Server version 1.0.1, Copyright 1997 Luca Pavanello and Marco Gaiarin");
    System.out.println("Domino Server comes with ABSOLUTELY NO WARRANTY;");
    System.out.println("This is free software, and you are welcome to redistribute it");
    System.out.println("under certain conditions; see GNU GPL version 2 or above for detail");
    System.out.println("");
    System.out.println("Usage:  java Domino [-l debuglevel] [-p port] [-f [logfile]] [-g maxgame]");
    System.out.println("");
    System.out.println("	-l	set debugging level 0...10 to debuglevel, default "+defLevel);
    System.out.println("		(minimal logging)");
    System.out.println("	-p	set TCP port to use (>"+ (minPort-1) +"), default "+defPort);
    System.out.println("	-g	set maxgame number of contemporary plays (>"+ (minGames-1) +", <"+ (MAXGames+1) +")" );
    System.out.println("		default "+ minGames);
    System.out.println("");
    System.out.println("Report bug to gaio@dei.unipd.it");
  
  }


}  // class Server
