import ServerLog;

// ***************************************************************************
//			ServerCounter
// ***************************************************************************

// Classe statica che si preoccupa di tenere traccia del numero di giochi
// correntemente attivi, attraverso due metodi che permettono l'aggiunta
// e la rimozione di un gioco attivo.
//
// Il metodo che incrementa i giochi segnala il raggiungimento di un numero
// massimo di giochi definito inizalmente, permettendo quindi di evitare di
// crearne altri
//
// la classe  synchro per rendere sicuro il numero di giochi correntemente
// in uso (classico esempio di incremento/decremento che va protetto da
// semaforo).


class ServerCounter {

	// numero massimo di giochi in esecuzione
  private static int	maxGames = -1;
  	// numero di giochi in esecuzione
  private static int	numGames = 0;



	// definisce il numero massimo di giochi, operazione che pu essere
	// svolta una volta sola

  public static synchronized void setMaxGames(int max) {
  
    if (maxGames < 0)
      if (max > 0)
	maxGames = max;
      else
      	maxGames = 16;	// 0 non  valido, uso 16 come default
  }


	// addGame aggiunge un gioco a quelli attivi, sempre se non ha
	// raggiunto il numero massimo di giochi nel qul caso ritorna -1
	//
	// Ritorna:
	// -1	se ci sono gi abbastanza giochi e non ne possono essere
	//	aggiunti altri
	//  0	se ho aggunto un gioco regolarmente

  public static synchronized int addGame() {

    if ( numGames >= maxGames )
      return -1;
    else {
      numGames++;
      return 0;
    }
  }


	// removeGame rimuove un gioco attivo dal contatore; se non c'erano
	// giochi attivi questo  considerato errore grave e si esce da tutto

  public static synchronized void removeGame() {
  
    if ( numGames > 0 )
      numGames--;
    else {
      ServerLog.doLog(0,"ServerCounter.java: code error, please debug");
      System.exit(1);
    }
  }

  
} // ServerCounter
