import java.awt.Point;
import java.io.*;
import java.net.*;
import ServerMap;
import ServerThread;
import ServerCounter;

// ***************************************************************************
//		ServerGame (synchro)
// ***************************************************************************

// Questa  la classe che, assieme a ServerMap e Map, gestisce a tutto gli
// effetti il gioco.
// Questa in particolare gestisce i thread a cui sono associati i client, e
// presenta tutti i metodi synchronized che servono ai thread per poter far
// sviluppare il gioco.
//
// in pratica funge da ``interfaccia'' verso il gioco vero (ServerMap e Map)
// gestendo le sincronizzazioni


public class ServerGame {

	// Puntatore alla classe Map
  ServerMap map;

	//  ServerThread attivo
  private int actThread;

  	// Numero corrente di giocatori e ServerThread
  private int		numPlayer = 0;
  private int		numThread = 0;

	// Numero massimo di ServerThread
  final public int MAXThread = 8;

	// Numero Massimo e minimo di giocatori per una partita
  final public int MAXPlayer = 5;
  final public int minPlayer = 2;

	// Lista di tutti i ServerThread
  private ServerThread 	threadList[];

	// Numero e nome del giocatore associato al ServerThread
  private int		playerList[];
  private String	nameList[];

	// Numero di questo gioco, usato soprattutto per debugging
	// e visualizzazione
  private int		numGame;



	// Costruttore, fa le dovute inizializzaioni

  public ServerGame(int numGame) {

    ServerLog.doLog(0,numGame,actThread,"ServerGame.init: game created");

	// inizializzazione dela istanza di Map che gestisce il gioco
    map = new ServerMap( new Point(14,14) );

	// inizializzazione delle variabili locali
    threadList	= new ServerThread[MAXThread];
    playerList	= new int[MAXThread];
    nameList	= new String[MAXThread];

    this.numGame = numGame;    

	// imposta i ServerThread in stato inattivo
    for (int i=0 ; i<MAXThread ; i++)
      playerList[i] = 0;

	// ci pensa getActiveThread a fare ulteriori controlli...
    actThread = 0;

 }


	// gestisce l'accettazione da un socket, creando un nuovo ServerThread
	// che gestisce la connessione
	// Restituisce :
	//
	//  0 : Socket accetato
	// -1 : Raggiunto il numero massimo di Thread
	// -2 : Partita Iniziata

  public synchronized int acceptSocket( Socket socket ) {

	// Numero massimo di Thread raggiunto
    if ( numThread == MAXThread )
      return -1;
      
	// Partita Iniziata
    if ( map.getState() != 0 )
      return -2;

    try {
      threadList[numThread] = new ServerThread ( numThread,socket,this );
    } catch (IOException e) {
	// una connessione non acettata non  un errore, basta ignorarlo ;)
      return 0;
    }

	// Viene generato e fatto partire il thread associato a ServerThread
    new Thread( threadList[numThread] ).start();
    ServerLog.doLog(3,numGame,numThread,"AcceptSocket: thread accepted");
    
       	// il numero dei thread viene incrementato
    numThread++;

    return 0;
  }



	// ritorna lo stato del ServerThread, ovvero:
	// 0	ServerThread attivo come giocatore e partita iniziata
	// 1	ServerThread attivo ma non presentatosi, quindi partita
	//	non ancora iniziata
	// 2	ServerThread defunto in partita
	// 3	ServerThread defunto e non presentatosi, non esistente o
	//	defunto prima dell'inizio della partita

private int threadState( int thread ) {

	// looks fine, eh? ;)
int b1 = ( threadList[thread] == null ) ? 2 : 0 ;
int b2 = ( playerList[thread] == 0 ) ? 1 : 0 ;

return b1+b2;

}



	// serve in ServerThread per un corretto logging

  public int getNumGame() {
  
    return numGame;
    
  }  



	//  un semplice wrapper per accedere a Map.getBlock passando da
	// ``thread'' a ``player''
	// Restituisce:
	//
	//   0 : Il pezzo appartiene al giocatore
	//  -1 : Posizione pezzo non valida
	//  -2 : il pezzo non appartiene al giocatore

  public int getBlock( int thread, Point block ) {
  
    int player = playerList[thread];

	// questo non dovrebbe MAI accadere, per quello che c' il
	// System.exit(1)!!!
    if ( threadState(thread) != 0 ) {
      ServerLog.doLog(0,numGame,thread,"ServerGame.getBlock: non active thread");
      System.exit(1);
    }

    return( map.getBlock( block , player ) );    

  }



	// va al successivo thread attivo, eseguendo tutta una serie di
	// controlli sullo stato della partita
	// Restituisce:
	// 
	//   0: la partita pu continuare
	//  -1: la partita  da terminare

  private int nextActiveThread() {

	// idem come sopra, qui c' un System.exit(1) perch questo codice
	// non dovrebbe essere mai eseguito, vedi sotto.
    if (numPlayer == 0) {
      ServerLog.doLog(0,numGame,actThread,"ServerGame.nextActiveThread: no active thread");
      System.exit(1);
    }

	//  inutile che uno giochi con se stesso, vince per abbandono...
    if (numPlayer == 1)
      return -1;

    int i;

	// cerco un giocatore che possa muovere
    for ( i=1;	(i < MAXThread+1) ; i++) 
      if ( threadState((actThread+i)%MAXThread) == 0 )
        if ( map.canMove(playerList[(actThread+i)%MAXThread]) == 0) break;

	// se ho trovato un giocatore valido...
    if ((threadState((actThread+i)%MAXThread) == 0 ) && ( map.canMove(playerList[(actThread+i)%MAXThread]) == 0))
      actThread = (actThread+i)%MAXThread;
    else {
    	// evidentemente ci sono almeno due giocatori, ma nessuno pu
    	// muovere... il gioco  FINITO!!! ;)))
      return -1;
    }

    return 0;

  }



	// libera il <player> giocatore che  morto a gioco non iniziato.
	// non essendo cominciato il gioco, possiamo tranquillamente
	// eliminarlo e ricompattare la lista dei giocatori.

  private void setFreePlayer(int thread) {

	// questo valore me lo devo salvare...
    int player = playerList[thread];

	// ricompatto i giocatori riassegnando i numeri
    for (int i=0; i < numThread; i++)
      if ( playerList[i] > player )
        playerList[i] = playerList[i]-1;

    playerList[thread] = 0;
    nameList[thread] = "";
     
  }
  
  
  

// ***************************************************************************
//			do*
// ***************************************************************************

// I metodi do*, tutti simili tra loro, sono i metodi synchro che eseguono
// tutte le operazioni sul gioco.
// Tra un metodo e l'altro cambiano molto le condizioni che vengono
// controllate (molto strette per taluni, lasche per altri...) e le operazioni
// compiute, ma concettualmente sono molto simili
//
// per renderli ancora pi simili  stato adottato il metodo threadState() e
// un case sui possibili stati
//
// le funzioni non ritornano nulla,  previsto esplicitamente che ogni comando
// non valido venga semplicemente ignorato, tuttalpi viene eseguita una riga
// di logging
//
// unica eccezione doDEAD che ritorna un valore != 0 se la partita  finita,
// e questo serve per uscire velocemente dagli altri cicli



	// Comunica il Nome a tutti i client e abilita il thread come
	// giocatore, operazione fondamentale alla fine della gestione del
	// gioco

  synchronized public void doNAME( int thread, String name) {
    
    ServerLog.doLog(10,numGame,thread,"S <- C : NAME "+name); 

    int player = playerList[thread];

    
    switch ( threadState(thread) ) {

      case 0:	// il ServerThread ha gi nome, ed  attivo. Non si pu
      		// ripresentare... questo ci evita anche il controllo sullo
      		// stato della partita, visto che solo per stato=1 facciamo
      		// qualcosa di notabile...
  
        ServerLog.doLog(4,numGame,thread,"ServerGame.doNAME: already defined NAME for player " + player);
        ServerLog.doLog(10,numGame,thread,"S -> C : "+player+".QUOTE 0,Nome gia' definito");

        try { 
	  threadList[thread].writeQUOTE(0, "Nome gia' definito"); 
        } catch (IOException e) {
          if ( doDEAD(thread,e) < 0) return;
        }
        break;

      case 1:	// il ServerThread  attivo, ma non si  ancora presentato.
      		//  il caso che ci interessa...

	// eseguiamo le operazioni di assegnazione del giocatore e poniamo
	// lo stato del thread a 0

        numPlayer++;
        player = numPlayer;
        playerList[thread] = player;
        nameList[thread] = new String(name);    

	// ogni volta che un giocatore si presenta, aggiorniamo
	// l'informazione per tutti i clients abilitati

	// una prima volta avviso il neogiocatore dello stato di tutti
        for ( int t=0 ; t < numThread ; t++ )
          if (threadState(t) == 0 )
	    try { 
              threadList[thread].writeNAME( playerList[t], nameList[t] );
	    } catch (IOException e) {
	      doDEAD(thread,e);
	      return;
	    }

	// una seconda volta avviso gli altri del nuovo arrivato
        for ( int t=0 ; t < numThread ; t++ )
          if ((threadState(t) == 0 ) && (t != thread ))
            try {
              threadList[t].writeNAME( playerList[thread], nameList[thread] );
            } catch (IOException e) {
              if ( doDEAD(t,e) < 0) return;
            }
    

	// Se ho raggiunto il numero massimo di giocatori, il gioco comincia
        if (numPlayer == MAXPlayer)
          doSTART(thread);
        
        break;

      case 2:	// il ServerThread  morto, e quindi non si pu presentare ;)
      case 3:	// non facciamo assolutamente nulla! ;)

        ServerLog.doLog(4,numGame,thread,"ServerGame.doNAME: dead ServerThread trying to greeting... ");
        break;
    }
  }



	// incomincia il gioco

  synchronized public void doSTART( int thread ) {

    int player = playerList[thread];
    String str = null;

    ServerLog.doLog(10,numGame,thread,"S <- C : START");
    
    switch ( threadState(thread) ) {
    
      case 0:	// il giocatore  buono, ha diritto di presentarsi...

	// ci deve essere almeno un minPlayer numero di giocatori
        if (numPlayer < minPlayer )	{
          ServerLog.doLog(4,numGame,thread,"ServerGame.doSTART: ignored, not enougth players");
          try { 
	    threadList[thread].writeQUOTE(0,"non ci sono abbastanza giocatori!"); 
          } catch (IOException e) {
            if ( doDEAD(thread, e) < 0) return;
          }
          return;
        }

	// la partita NON deve essere iniziata!
        if ( map.getState() != 0 ) {
          ServerLog.doLog(4,numGame,thread,"ServerGame.doSTART: ignored, game already started");
          try { 
	    threadList[thread].writeQUOTE(0,"il gioco  gi cominciato"); 
          } catch (IOException e) {
            if ( doDEAD(thread, e) < 0) return;
          }
          return;
        }

	// Ok, sembra che questo sia uno START valido...

        ServerLog.doLog(0,numGame,thread,"ServerGame.doSTART: game started");

	// Uccido i thread che si sono collegati ma non presentati
        for (int t=0 ; t < numThread ; t++ )
          if ( threadState(t) == 1 ) {
            threadList[t].disableThread();
            threadList[t]=null;
          }

        ServerLog.doLog(10,numGame,thread,"Thread attivi: "+ Thread.activeCount());
        for (int i=0; i < numThread; i++) {
          ServerLog.doLog(10,numGame,thread,"thread " + i + " e' giocatore " + playerList[i] + " di " + numPlayer + " giocatori");
          if (threadList[i]==null) ServerLog.doLog(10,numGame,thread,"thread " + i + "NULL");
        } 
        ServerLog.doLog(10,numGame,thread,"Numero Giocatori prima di giveBlocks() : "+numPlayer);

	// Diamo inizio alla partita

	// ``The START fever'', il primo che da start inizia il gioco!
        actThread=thread;

	// Distribuzione pezzi
	// la giveBlocks si preoccupa anche di cambiare stato al gioco
        map.giveBlocks(numPlayer);

	// questo mi serve per evitare che la morte di un giocatore nel ciclo
	// a seguire provochi la spedizione di dati inesatti; questo perch
	// numPlayer rappresenta il numero di giocatori vivi e attivi, mentre
	// al client interessa il numero di giocatori vivi allo START
	int np = numPlayer;

	// Avvisiamo i clients di esistere (in quanti giocatori) e chi sono
	// (numero di giocatore)
        for (int t=0 ; t < numThread ; t++ )
          if (threadState(t) == 0 )
	    try {
	      threadList[t].writePLAY(playerList[t],np);
	    } catch (IOException e) {
	      if ( doDEAD(t,e) < 0) return;
	    }

	// Trasmissione pezzi ai clients
        for ( int t=0 ; t < numThread ; t++ ) 
          if (threadState(t) == 0)
	    try {
              threadList[t].writeAllBlocks(map.numblock);       
              threadList[t].writeSTART();
	    } catch (IOException e) {
	      if ( doDEAD(t,e) < 0 ) return;
            }

      	// il giocatore attivo viene abilitato a muovere...
	// l'informazione actThread  sempre congruente e aggiornata, visto
	// che in doDEAD viene fatto il controllo e l'aggiornamento...
        try { 
          threadList[actThread].writeGO();
        } catch (IOException e) {
          if ( doDEAD(actThread,e) < 0 ) return;
        }
        
        break;
        
      case 1:	// Il giocatore deve essere stato abilitato

        ServerLog.doLog(4,numGame,thread,"ServerGame.doSTART: ignored, thread disabled");
        break;

      case 2:	// giocatore morto non parla!
      case 3:

        ServerLog.doLog(4,numGame,thread,"ServerGame.doSTART: ignored, thread dead");
        break;
        
    }
  }



	// esegue la mossa aggiornando lo stato del gioco nel server e
	// comunicando la mossa a tutti i client.
   
  synchronized public void doMOVE( int thread, int blk1, int blk2, int x, int y, int mode ) {

    int player = playerList[thread];

    ServerLog.doLog(10,numGame,thread,"S <- C : MOVE "+blk1+","+blk2+","+x+","+y+","+mode);


    switch ( threadState(thread) ) {

      case 0:	// sembra che il ServerThread sia uno di quelli buoni...

	// non  il suo turno...
        if ( thread != actThread ) {
          ServerLog.doLog(4,numGame,thread,"ServerGame.doMOVE: ignored, not your turn"); 
          try { 
	    threadList[thread].writeQUOTE(0, "Mossa non valida: non  il tuo turno"); 
          } catch (IOException e) {
            if ( doDEAD(thread,e) < 0) return;
          }
          return;
        }

	// proviamo a fare la mossa chiamando la funzione di ServerMap...
        int err = map.putMap( player, new Point(blk1,blk2), new Point(x,y), mode );

	// c' un errore!
        if ( err != 0 ) {
          ServerLog.doLog(4,numGame,thread,"ServerGame.doMOVE: ignored, reason " + map.errorString(err));
          ServerLog.doLog(10,numGame,thread,"S -> C : "+player+".QUOTE 0,Mossa non Valida");
          try { 
	    threadList[thread].writeQUOTE(0, "Mossa non Valida: "+ map.errorString(err)); 
          } catch (IOException e) {
            if ( doDEAD(thread,e) < 0) return;
          }
          return;
        }

	// OK, la mossa  buona e quindi possiamo fare tutte le operazioni di
	// routine

	// Aggiorno giocatore attivo; lo faccio qui perch se nel for
	// successivo per caso morisse il giocatore correntemente attivo che ha
	// appena mosso, la chiamerei due volte.
	// A questo punto ci devono essere almeno due giocatori, altrimenti
	// sarei gi uscito in qualche doDEAD->theEND
        int flag = nextActiveThread();

	// potrebbe essere anche successo che con questa mossa il giocatore ha
	// finito i pezzi. TU HAI.. TU HAI.. TU HAI FINTO!!!
        if (map.getNumBlockPlayer(player) == 0)
          flag = -1;

	// Trasmissione mossa ai client 
        for ( int t=0 ; t< numThread ; t++ ) 
          if ( threadState(t) == 0 )
            try { 
	      threadList[t].writeMOVE(player, blk1, blk2, x, y, mode); 
            } catch (IOException e) {
              if ( doDEAD(t,e) < 0) return;
            }

	// In ogni caso Il giocatore di turno viene abilitato a muovere, a
	// meno che la partita non sia finta, in quel caso esco
        if ( flag == 0 )
          try {
            threadList[actThread].writeGO();
          } catch (IOException e) {
            if ( doDEAD(actThread,e) < 0) return;
          }
        else
          theEND(); // this is the END, oh my baby, this is the END...
          
        break;
        
      case 1:	// Se e' un Giocatore non abilitato a muovere

        ServerLog.doLog(4,numGame,thread,"ServerGame.doMOVE: ignored, thread disabled");
        break;
        
      case 2:	// ServerThread morto non parla...
      case 3:

        ServerLog.doLog(4,numGame,thread,"ServerGame.doMOVE: ignored, thread dead");
        break;

    }
  }



	// comunica il QUOTE a tutti i client.

  synchronized public void doQUOTE( int thread, String quote) {

    int player = playerList[thread];

    ServerLog.doLog(10,numGame,thread,"S -> C : QUOTE: "+player+","+quote);


    switch ( threadState(thread) ) {

      case 0:	// OK, il QUOTE  buono, non ci sono altri controlli da fare...

    	// mando il QUOTE a tutti gli altri giocatori attivi
        for ( int t=0 ; t< numThread ; t++ )
          if ( threadState(t) == 0 ) 
            try { 
  	      threadList[t].writeQUOTE(player, quote); 
	    } catch (IOException e) {
              if ( doDEAD(t,e) < 0) return;
            }
        break;

      case 1:	// il giocatore deve essere attivo

        ServerLog.doLog(4,numGame,thread,"ServerGame.doQUOTE: ignored, thread disabled");
        break;

      case 2:	// ServerThread morto non parla...
      case 3:

        ServerLog.doLog(4,numGame,thread,"ServerGame.doQUOTE: ignored, thread dead");
        break;

    }
  }




	// Uccide un thread, per il motivo ``e''
	// qualcosa e' andato storto nella scrittura del socket, oppure e'
	// venuto il tempo di uccidere i thread in piu' , e quindi
	// devo uccidere anche il giocatore in questione
	//
	// Ritorna:	0 :	Tutto a posto, giocatore ucciso
	//		-1:	Partita finita (theEND a questo punto 
	//			gi stato chiamato...

  public synchronized int doDEAD( int thread , Exception e) {

    int player = playerList[thread];

    ServerLog.doLog(1,numGame,thread,"ServerGame.doDEAD: thread closed, reason "+ e);

    switch ( threadState(thread) ) {

      case 0:	// questo  un ServerThread buono, da uccidere con creanza...

	// per uccidere il thread semplicemente setto un flag che fa uscire
	// dal suo metodo run, e pongo il puntatore a null. Appena esco
	// dalle chiamate innestate l'oggetto dovrebbe essere automaticamente
	// GarbageCollected (e in effetti sembra che accada...)
        threadList[thread].disableThread();
        threadList[thread]=null;
        numPlayer--;
        
        // se la partita  in corso devo semplicemente avvisare gli altri.
        // lo stato del ServerThread  gi stato posto a 2 dall'istruzione
        // precedente
        if ( map.getState() != 0 ) {

		// riscriviamo il nome con qualcosa di esplicativo
	  nameList[thread] = nameList[thread] + " (morto) ";

		// Gestione fine partita; se  rimasto un giocatore solo...
          if ( numPlayer == 1 ) {
            theEND();
            return -1;
          }

    		// ora avvisiamo gli altri... non posso usare doNAME, dovrei
    		// aggirare il controllo che ho messo...
    		// inoltre aggiungo il punteggio che gi conosciamo...
          for ( int i=0 ; i < numThread ; i++ )
            if ( threadState(i) == 0 )
              try {
                threadList[i].writeNAME( player, nameList[thread] + " " + map.getNumBlockPlayer(player));
	      } catch (IOException err) {
	        if (doDEAD(i,err) < 0) return -1;
	      }

		// Se  morto il giocatore corrente...
          if ( thread == actThread ) 
		// Se la partita  finita...
            if (nextActiveThread() < 0 ) {		
              theEND();
              return -1;
          	// Altrimenti abilito il giocatore seguente; se anche questo
          	// muore richiamo doDEAD e la cosa viene gestita automagically
            } else 
              try {
	        threadList[actThread].writeGO(); 
	      } catch (IOException err) {
	        return doDEAD(actThread,err);
	      }

	// la partita non  iniziata, possiamo essere un pelo pi rilassati..
        } else {

		// ricompatto la lista dei giocatori, eliminando lo sfigato
		// che  morto prima di vedere l'inizio della partita...
          setFreePlayer(thread);

    		// ora poniamo in uno stato preciso i client... non posso
    		// usare doNAME, dovrei aggirare il controllo che ho messo...
          for ( int i=0 ; i < numThread ; i++ )
	    if (threadState(i) == 0)
	      try { 
                threadList[i].writeCLEAN( player );
	      } catch (IOException err) {
	 	if (doDEAD(i,err) < 0) return -1;
              }

	}
	
	break;
	
      case 1:	// il ServerThread  vivo ma non si  presentato...
      		// basta solo ucciderlo fisicamente

        threadList[thread].disableThread();
        threadList[thread]=null;
        
        break;

        
      case 2:	// ServerThread morto non si uccide due volte... ;)
      case 3:

        ServerLog.doLog(4,numGame,thread,"ServerGame.doDEAD: ignored, thread already dead");
      
    }

    return 0;

  }



	// sembra quasi che il gioco sia finito, prendo le naturali
	// conseguenze...
	//
	// questa non  synchro, non serve visto che viene chiamata dalle
	// altre do* (doDEAD e doMOVE), e non deve fare molti controlli
	// visto che per ipotesi la partita  iniziata (altrimenti come fa a
	// finire? ;)
	

  public void theEND() {


	// per prima cosa avviso tutti i giocatori rimasti in vita del loro
	// punteggio e di quello degli altri (stati 0 e 2, anche se per quelli
	// a stato 2 dovrebbe averci gi pensato doDEAD...)
    for ( int t=0 ; t < numThread ; t++ )
      if (threadState(t) == 0)
        for (int i=0; i < numThread; i++)
          if ( (threadState(i) == 0) || (threadState(i) == 2) )
            try {
              threadList[t].writeNAME( playerList[i], nameList[i] + " " + map.getNumBlockPlayer(playerList[i]) ); }
            catch (IOException e) {}


	// ora un bel repulisti; come si pu notare il trattamento  veloce
	// e indolore...
    for ( int t=0 ; t < numThread ; t++ )
      if (threadState(t) == 0) {
        try {
          threadList[t].writeEND(); }
        catch (IOException e) {}
        threadList[t].disableThread();
        threadList[t]=null;
	playerList[t]=0;
      }

	// segnalo l'avvenimento
    ServerLog.doLog(0,numGame,actThread,"ServerGame.theEND: game terminated");

  }



  public void finalize() {

	// solo una riga di debugging e elimino il gioco dal contatore
    ServerLog.doLog(10,numGame,actThread,"ServerGame.finalize: game disposed");
    ServerCounter.removeGame();
    
  }


} // ServerGame
