import java.net.*;
import java.io.*;
import java.util.Date;
import java.text.SimpleDateFormat;


// ***************************************************************************
//			ServerLog
// ***************************************************************************

// Classe statica (incredibile ma vero, abbiamo scoperto l'utilit delle
// classi statiche ;) che si preoccupa di eseguire il logging in una forma
// che sia UNIX-like, ovvero:
//
// mese giorno ora hostname SDomino[game](thread): stringa di log
//
// dove game e thread rappresentano il numero di game e il numero di
// ServerThread che ha generato la riga di log
//
// I livelli di log sono cos definiti:
//
// 0:	partenza, errori fatali, creazione, inizio e terminazione dei giochi
// 1:	chiusura forzata dei thread (doDEAD)
// 2:	apertura e chiusura dei socket
// 3:	accettazione dei thread
// 4:	condizioni di errore varie nelle do*
// 5:	condizioni di errore nel parser
// 6-9:	non definiti
// 10:	debugging
//
// il livello di default  0.
//
// la classe  synchro per rendere sicura la definizione del loglevel e per
// rendere leggibile il logging, che altrimente rischierebbe di essere
// sovrapposto.


class ServerLog {

  private static int	logLevel = -1;
  private static String	localhost;


	// definisce i livello di logging, operazione che pu essere svolta
	// una volta sola

  public static synchronized void setLevel(int level) {
  
    if (logLevel < 0)
      if (level >= 0)
        if (level <= 10) {
        
          String lh;
        
          logLevel = level;
          
          try {
            lh = InetAddress.getLocalHost().getHostName();
          }
          catch (Exception e) {
            localhost = "unknown";
            return;
          }
          
          if ( lh.indexOf(".") <= 0 )  localhost=lh;
	  else localhost = lh.substring(0,lh.indexOf("."));
       
        }  
  }


	// versione per Server

  public static synchronized void doLog ( int l, String s) {

    if (l <= logLevel) {
      SimpleDateFormat format = new SimpleDateFormat("MMM dd hh:mm:ss");

      System.out.println(format.format(new Date()) + " " + localhost + " SDomino: " + s);
    }
  }
  


	// versione per tutti gli altri, in pi riporta l'informazione di
	// game e thread

  public static synchronized void doLog ( int l, int g, int t, String s) {

    if (l <= logLevel) {
      SimpleDateFormat format = new SimpleDateFormat("MMM dd hh:mm:ss");
    
      System.out.println(format.format(new Date()) + " " + localhost + " SDomino["+g+"]("+t+"): " + s);
    }
  }

  
} // ServerLog
