import java.awt.Point;
import ServerLog;
import Map;

// *****************************************************************************
//		ServerMap
// *****************************************************************************

// Implementa le funzione del gioco riservate al server.
// Fornisce le primitive per :
// 
//  1 : Distribuire i pezzi ai giocatori 
//  2 : Decidere quale giocatore deve muovere
//  3 : Gestire fine partita e punteggi


class ServerMap extends Map {


  public ServerMap( Point size ) {

	// Costruttore, chiama il costruttore della classe Map

    super ( size );
  }


  public void giveBlocks( int numPlayer ) {

	// Distribuisce i pezzi ai giocatori e da inizio alla partita

	// La distribuzione dei pezzi deve essere casuale. Viene generato un 
	// numero 'n' Random nell' intervallo 0..Numero pezzi ancora da Assegnare-1
	// Seguento un ordine fra i pezzi viene estratto n-esimo libero.  
	// Scelto un giocatore alla volta utilizzando questa procedura e' possibile
	// assegnarli i pezzi.

    if ( getState() != 0 ) {

	// Se la partita e' iniziata vuol dire che questa procedura e' gia' 
	// stata eseguita.......errore nel codice.  
      ServerLog.doLog(0,"ServerMap.giveBlock : giveBlock richiamata due volte");
      System.exit(1);
    }
    setNumPlayer( numPlayer );		// Imposta il numero di giocatori

    int totblock = numblock*(numblock+1)/2;
    int db = totblock / numPlayer;	// Pezzi per giocatore
    int mb = totblock % numPlayer;	// Giocatori a cui deve essere assegnato
					// un pezzo in piu'
    int i,j,np;
    Point pt;

	// Ciclo su tutti i giocatori e sul numero di pezzi da assegnare al giocatore 
    for ( i=1 ; i<=numPlayer ; i++ ) 	
      for ( j=1 ; !( (( j>db+1 )&&( i<=mb )) || (( j>db )&&( i>mb)) ) ; j++ ) {

		// numero Random nell'intervallo 0..Numero pezzi ancora da Assegnare-1.
        np = (int)(totblock*Math.random());
        if (np==totblock)  np--;
		// np-esimo pezzo ancora da assegnare
        pt = getFreeBlock( np );
        if  ( putBlock(pt ,i) != 0 ) {
		// Pezzo da assegnare gia' assegnato....errore nel codice
	  ServerLog.doLog(0,"ServerMap.givesBlocks : Errore Ingestibile");
          System.exit(1);
	}
        totblock--;	// Numero Pezzi ancora da assegnare --
    }
    startPlay();	// Da inizio alla partita
  } 


  private Point getFreeBlock( int nb ) {    

	// Restituisce l' nb-esimo Pezzo libero

    int i,j;	// Indici
    Point pt = new Point(0,0);

	// I pezzi vengono scanditi per righe ( 'i' indica la riga 'j' la colonna )

    for ( i=0 ; i<numblock ; i++ ) 
      for ( j=0 ; j<=i ; j++ ) {
        pt.move(i,j);
        switch ( getBlock( pt,0) ) {
	case 0 :
		// Pezzo ancora da assegnare.
		// Se 'nb' e' uguale a zero 'pt' rappresenta il pezzo cercato
		// altrimenti 'nb' viene decrementato 
          if ( nb==0 ) return pt;
          else nb--;
	  break;
	case -1 :
		// Chiamata non corretta a getBlock()....errore nel codice
	  ServerLog.doLog(0,"ServerMap.getFreeBlock : Errore Ingestibile");
          System.exit(1);
		// Se il pezzo e' gia assegnato si passa al pezzo successivo
	}        
      }         
	// l'nb-esimo pezzo libero non e' stato trovato....errore nel codice
    ServerLog.doLog(0,"ServerMap.getFreeBlock : Errore Ingestibile");
    System.exit(1);
    return null;
  }    


  public int canMove( int player ) {

	// Restituisce  :
	//
	//   0 : Il giocatore player puo' muovere 
	//  -1 : Il giocatore player non puo' muovere 

    int	rtr = -1;	//  -1 il giocatore non puo' muovere, 
			//   0 il giocatore puo' muovere
    Point pt = new Point(0,0);

	// Il giocatore 'player' puo' muovere se possiede un pezzo (e1,i) o (e2,i) 
	// con e1,e2 le figure nei due estremi e i appartenente a 0..numblock-1
	
    for ( int i=0 ; ( i < numblock ) && ( rtr == -1 ) ; i++ ) {

      pt.move( getExtreme(1),i );
      switch ( getBlock( pt,player )) {
      case 0 : 
	rtr=0;		// Il giocatore possiede il pezzo (e1,i)
	break;
      case -1 :
		// Chiamata non corretta a getBlock()....errore nel codice
	ServerLog.doLog(0,"ServerMap.canMove : Errore Ingestibile");
        System.exit(1);
      }

      pt.move( getExtreme(2),i );
      switch ( getBlock( pt,player ) ) {
      case 0 :
	rtr=0;		// Il giocatore possiede il pezzo (e2,i)
	break;
      case -1 :
		// Chiamata non corretta a getBlock()....errore nel codice
	ServerLog.doLog(0,"ServerMap.canMove : Errore Ingestibile");
        System.exit(1);
      }
    }
    return rtr;
  }


  public int getNumBlockPlayer( int player ) {    

	// Restituisce il numero di pezzi posseduti dal giocatore 'player'

    int i,j;	// indici
    int n;	// numero di pezzi posseduti dal giocatore
    Point pt = new Point(0,0);

	// I pezzi vengono scanditi per righe ( 'i' indica la riga 'j' la colonna )
 
    for ( i=0 , n=0 ; i<numblock ; i++ ) 
      for ( j=0 ; j<=i ; j++ ) {
        pt.move(i,j);
        switch ( getBlock( pt,player) ) {
	case 0 :
		// Il giocatore possiede il pezzo
	  n++;
	  break;
	case -1 :
		// Chiamata non corretta a getBlock()....errore nel codice
	  ServerLog.doLog(0,"ServerMap.getNumBlockPlayer : Errore Ingestibile");
          System.exit(1);
	}
      } 
    return n;        
  }    


}  // class


